// j01.loading.less
#j01Loading {
    position: fixed;
    z-index: 999999999;
    top: 85px;
    left: -6px;
    width: 1%;
    height: 2px;
    background: #b91f1f;
    -moz-transition: all 500ms ease-in-out;
    -ms-transition: all 500ms ease-in-out;
    -o-transition: all 500ms ease-in-out;
    -webkit-transition: all 500ms ease-in-out;
    transition: all 500ms ease-in-out;
    dt {
        width: 180px;
        right: -80px;
        clip: rect(-6px,90px,14px,-6px);
    }
    dd {
        width: 20px;
        right: 0;
        clip: rect(-6px,22px,14px,10px);
    }
    dd,
    dt {
        opacity: .6;
        position: absolute;
        top: 0;
        height: 2px;
        border-radius: 100%;
    }
    &.waiting dd,
    &.waiting dt {
        -moz-animation: pulse 2s ease-out 0s infinite;
        -ms-animation: pulse 2s ease-out 0s infinite;
        -o-animation: pulse 2s ease-out 0s infinite;
        -webkit-animation: pulse 2s ease-out 0s infinite;
        animation: pulse 2s ease-out 0s infinite;
    }
}

@keyframes pulse {
    30% { opacity:0.6; }
    60% { opacity:0; }
    100% { opacity:0.6; }
}

@-moz-keyframes pulse {
    30% { opacity:0.6; }
    60% { opacity:0; }
    100% { opacity:0.6; }
    }

@-ms-keyframes pulse {
    30% { opacity:0.6; }
    60% { opacity:0; }
    100% { opacity:0.6; }
}

@-webkit-keyframes pulse {
    30% { opacity:0.6; }
    60% { opacity:0; }
    100% { opacity:0.6; }
}
